@@ -15,6 +15,7 @@ from geneticengine.algorithms.callbacks.callback import ProgressCallback
 from geneticengine.algorithms.callbacks.csv_callback import CSVCallback
 from geneticengine.algorithms.gp.individual import Individual
 from geneticengine.algorithms.heuristics import Heuristics
+from geneticengine.core.grammar import EvolveGrammar
 from geneticengine.core.grammar import Grammar
 from geneticengine.core.problems import FitnessType
 from geneticengine.core.problems import Problem
@@ -27,51 +28,49 @@ from geneticengine.core.representations.tree.treebased import treebased_represen
 
 
 class GP(Heuristics):
-    """
-    Genetic Programming object. Main attribute: evolve
+    """Represents the Genetic Programming algorithm. Defaults as given in A
+    Field Guide to GP, p.17, by Poli and Mcphee:
 
-    Parameters:
-        - grammar (Grammar): The grammar used to guide the search.
-        - representation (Representation): The individual representation used by the GP program. The default is treebased_representation.
-        - problem (Problem): The problem we are solving. Either a SingleObjectiveProblem or a MultiObjectiveProblem.
-        - evaluation_function (Callable[[Any], float]): The fitness function. Should take in any valid individual and return a float. The default is that the higher the fitness, the more applicable is the solution to the problem. Turn on the parameter minimize to switch it around.
-        - minimize (bool): When switch on, the fitness function is reversed, so that a higher result from the fitness function corresponds to a less fit solution (default = False).
-        - target_fitness (Optional[float]): Sets a target fitness. When this fitness is reached, the algorithm stops running (default = None).
-        - favor_less_deep_trees (bool): If set to True, this gives a tiny penalty to deeper trees to favor simpler trees (default = False).
-        - randomSource (Callable[[int], RandomSource]): The random source function used by the program. Should take in an integer, representing the seed, and return a RandomSource.
-        - population_size (int): The population size (default = 200). Apart from the first generation, each generation the population is made up of the elites, novelties, and transformed individuals from the previous generation. Note that population_size > (n_elites + n_novelties + 1) must hold.
-        - n_elites (int): Number of elites, i.e. the number of best individuals that are preserved every generation (default = 5).
-        - n_novelties (int): Number of novelties, i.e. the number of newly generated individuals added to the population each generation. (default = 10).
-        - number_of_generations (int): Number of generations (default = 100).
-        - max_depth (int): The maximum depth a tree can have (default = 15).
-        - selection_method (Tuple[str, int]): Allows the user to define the method to choose individuals for the next population (default = ("tournament", 5)).
-        -----
-        Default as given in A Field Guide to GP, p.17, by Poli and Mcphee:
-        - probability_mutation (float): probability that an individual is mutated (default = 0.01).
-        - probability_crossover (float): probability that an individual is chosen for cross-over (default = 0.9).
-        - either_mut_or_cro (float | None): Switch evolution style to do either a mutation or a crossover. The given float defines the chance of a mutation. Otherwise a crossover is performed. (default = None),
-        - hill_climbing (bool): Allows the user to change the standard mutation operations to the hill-climbing mutation operation, in which an individual is mutated to 5 different new individuals, after which the best is chosen to survive (default = False).
-        - specific_type_mutation (type): Specify a type that is given preference when mutation occurs (default = None),
-        - specific_type_crossover (type): Specify a type that is given preference when crossover occurs (default = None),
-        - depth_aware_mut (bool): If chosen, mutations are depth-aware, giving preference to operate on nodes closer to the root. (default = True).
-        - depth_aware_co (bool): If chosen, crossovers are depth-aware, giving preference to operate on nodes closer to the root. (default = True).
-        -----
-        - force_individual (Any): Allows the incorporation of an individual in the first population (default = None).
-        - seed (int): The seed of the RandomSource (default = 123).
-        -----
-        - timer_stop_criteria (bool): If set to True, the algorithm is stopped after the time limit (default = 60 seconds). Then the fittest individual is returned (default = False).
-        - timer_limit (int): The time limit of the timer.
-        -----
-        - save_to_csv (str): Saves a CSV file with the details of all the individuals of all generations.
-        - save_genotype_as_string (bool): Turn this off if you don't want to safe all the genotypes as strings. This saves memory and a bit of time.
-        - test_data (Any): Give test data (format: (X_test, y_test)) to test the individuals on test data during training and save that to the csv (default = None).
-        - only_record_best_inds (bool): Specify whether one or all individuals are saved to the csv files (default = True).
-        -----
-        - callbacks (List[Callback]): The callbacks to define what is done with the returned prints from the algorithm (default = []).
+    Args:
+        grammar (Grammar): The grammar used to guide the search.
+        representation (Representation): The individual representation used by the GP program. The default is treebased_representation.
+        problem (Problem): The problem we are solving. Either a SingleObjectiveProblem or a MultiObjectiveProblem.
+        evaluation_function (Callable[[Any], float]): The fitness function. Should take in any valid individual and return a float. The default is that the higher the fitness, the more applicable is the solution to the problem. Turn on the parameter minimize to switch it around.
+        minimize (bool): When switch on, the fitness function is reversed, so that a higher result from the fitness function corresponds to a less fit solution (default = False).
+        target_fitness (Optional[float]): Sets a target fitness. When this fitness is reached, the algorithm stops running (default = None).
+        favor_less_deep_trees (bool): If set to True, this gives a tiny penalty to deeper trees to favor simpler trees (default = False).
+        randomSource (Callable[[int], RandomSource]): The random source function used by the program. Should take in an integer, representing the seed, and return a RandomSource.
+        population_size (int): The population size (default = 200). Apart from the first generation, each generation the population is made up of the elites, novelties, and transformed individuals from the previous generation. Note that population_size > (n_elites + n_novelties + 1) must hold.
+        n_elites (int): Number of elites, i.e. the number of best individuals that are preserved every generation (default = 5).
+        n_novelties (int): Number of novelties, i.e. the number of newly generated individuals added to the population each generation. (default = 10).
+        number_of_generations (int): Number of generations (default = 100).
+        max_depth (int): The maximum depth a tree can have (default = 15).
+        max_init_depth (int): The maximum depth a tree can have in the initialisation population. Currently only working for treebased representation (default = max_depth).
+        selection_method (Tuple[str, int]): Allows the user to define the method to choose individuals for the next population (default = ("tournament", 5)).
+        ramped_half_and_half (bool): Specify whether you want the population to be initialized ramped half and half. Currently only working for treebased representation (default = True).
+        
 
+        probability_mutation (float): probability that an individual is mutated (default = 0.01).
+        probability_crossover (float): probability that an individual is chosen for cross-over (default = 0.9).
+        either_mut_or_cro (float | None): Switch evolution style to do either a mutation or a crossover. The given float defines the chance of a mutation. Otherwise a crossover is performed. (default = None),
+        hill_climbing (bool): Allows the user to change the standard mutation operations to the hill-climbing mutation operation, in which an individual is mutated to 5 different new individuals, after which the best is chosen to survive (default = False).
+        specific_type_mutation (type): Specify a type that is given preference when mutation occurs (default = None),
+        specific_type_crossover (type): Specify a type that is given preference when crossover occurs (default = None),
+        depth_aware_mut (bool): If chosen, mutations are depth-aware, giving preference to operate on nodes closer to the root. (default = True).
+        depth_aware_co (bool): If chosen, crossovers are depth-aware, giving preference to operate on nodes closer to the root. (default = True).
 
+        force_individual (Any): Allows the incorporation of an individual in the first population (default = None).
+        seed (int): The seed of the RandomSource (default = 123).
 
+        timer_stop_criteria (bool): If set to True, the algorithm is stopped after the time limit (default = 60 seconds). Then the fittest individual is returned (default = False).
+        timer_limit (int): The time limit of the timer.
 
+        save_to_csv (str): Saves a CSV file with the details of all the individuals of all generations.
+        save_genotype_as_string (bool): Turn this off if you don't want to safe all the genotypes as strings. This saves memory and a bit of time.
+        test_data (Any): Give test data (format: (X_test, y_test)) to test the individuals on test data during training and save that to the csv (default = None).
+        only_record_best_inds (bool): Specify whether one or all individuals are saved to the csv files (default = True).
+
+        callbacks (List[Callback]): The callbacks to define what is done with the returned prints from the algorithm (default = []).
     """
 
     # reason for union with noreturn in evaluation function, elitism and elitism: https://stackoverflow.com/questions/51811024/mypy-type-checking-on-callable-thinks-that-member-variable-is-a-method
@@ -111,17 +110,17 @@ class GP(Heuristics):
         ] = None,  # DEPRECATE in the next version
         minimize: bool = False,  # DEPRECATE in the next version
         target_fitness: float | None = None,  # DEPRECATE in the next version
+        evolve_grammar: EvolveGrammar | None = None,
         favor_less_deep_trees: bool = False,  # DEPRECATE in the next version
         randomSource: Callable[[int], RandomSource] = RandomSource,
         population_size: int = 200,
-        n_elites: int = 5,  # Shouldn't this be a percentage of population size?
+        n_elites: int = 5,
         n_novelties: int = 10,
         number_of_generations: int = 100,
         max_depth: int = 15,
-        # now based on depth, maybe on number of nodes?
-        # selection-method is a tuple because tournament selection needs to receive the tournament size
-        # but lexicase does not need a tuple
+        max_init_depth: int | None = None,
         selection_method: tuple[str, int] = ("tournament", 5),
+        ramped_half_and_half: bool = False,
         # -----
         # As given in A Field Guide to GP, p.17, by Poli and Mcphee
         probability_mutation: float = 0.01,
@@ -136,7 +135,7 @@ class GP(Heuristics):
         force_individual: Any = None,
         seed: int = 123,
         # -----
-        timer_stop_criteria: bool = False,  # TODO: This should later be generic
+        timer_stop_criteria: bool = False,
         timer_limit: int = 60,
         # -----
         save_to_csv: str = None,
@@ -144,7 +143,7 @@ class GP(Heuristics):
         test_data: Callable[
             [Any],
             float,
-        ] = None,  # TODO: Should be part of Problem Class  [LEON]
+        ] = None,
         only_record_best_inds: bool = True,
         # -----
         callbacks: list[Callback] = None,
@@ -163,6 +162,12 @@ class GP(Heuristics):
         self.population_size = population_size
         self.elitism = selection.create_elitism(n_elites)
         self.max_depth = max_depth
+        if max_init_depth:
+            assert max_init_depth <= self.max_depth
+            self.max_init_depth = max_init_depth
+        else:
+            self.max_init_depth = self.max_depth
+        self.evolve_grammar = evolve_grammar
         self.favor_less_deep_trees = favor_less_deep_trees
         self.novelty = selection.create_novelties(
             self.create_individual,
@@ -219,6 +224,7 @@ class GP(Heuristics):
         else:
             self.selection = lambda r, ls, n: [x for x in ls[:n]]
         self.force_individual = force_individual
+        self.ramped_half_and_half = ramped_half_and_half
 
         if save_to_csv:
             self.test_data = test_data
@@ -243,16 +249,17 @@ class GP(Heuristics):
             self.callbacks.append(c)
 
     def evolve(self, verbose=1) -> tuple[Individual, FitnessType, Any]:
-        """
-        The main function of the GP object. This function runs the GP algorithm over the set number of generations, evolving better solutions
+        """The main function of the GP object. This function runs the GP
+        algorithm over the set number of generations, evolving better
+        solutions.
 
-        Parameters:
-            - verbose (int): Sets the verbose level of the function (0: no prints, 1: print progress, or 2: print the best individual in each generation).
+        Args:
+            verbose (int): Sets the verbose level of the function (0: no prints, 1: print progress, or 2: print the best individual in each generation).
 
         Returns a tuple with the following arguments:
-            - individual (Individual): The fittest individual after the algorithm has finished.
-            - fitness (float): The fitness of above individual.
-            - phenotype (Any): The phenotype of the best individual.
+            individual (Individual): The fittest individual after the algorithm has finished.
+            fitness (float): The fitness of above individual.
+            phenotype (Any): The phenotype of the best individual.
         """
         self.callbacks = [
             cb
@@ -272,7 +279,7 @@ class GP(Heuristics):
             self.callbacks.append(ProgressCallback())
 
         # TODO: This is not ramped half and half
-        population = self.init_population()
+        population = self.init_population(self.ramped_half_and_half)
         if self.force_individual is not None:
             population[0] = Individual(
                 genotype=relabel_nodes_of_trees(
@@ -284,18 +291,21 @@ class GP(Heuristics):
         gen = 0
         start = time.time()
 
+        if self.evolve_grammar:
+            best_overall = True
+
         while (not self.timer_stop_criteria and gen < self.number_of_generations) or (
             self.timer_stop_criteria and (time.time() - start) < self.timer_limit
         ):
+
             npop = self.novelty(self.n_novelties)
-            npop.extend(
-                self.elitism(
-                    population,
-                    self.problem,
-                    self.get_best_individual,
-                    self.evaluate,
-                ),
+            elites = self.elitism(
+                population,
+                self.problem,
+                self.get_best_individual,
+                self.evaluate,
             )
+            npop.extend(elites)
             spotsLeft = self.population_size - len(npop)
             while spotsLeft > 0:
                 if self.either_mut_or_cro:
@@ -331,6 +341,26 @@ class GP(Heuristics):
             for cb in self.callbacks:
                 cb.process_iteration(gen + 1, population, time=time_gen, gp=self)
 
+            if self.evolve_grammar:
+                if best_overall:
+                    inds = elites
+                else:
+                    for e in elites:
+                        npop.remove(e)
+                    inds = npop
+                best = self.get_best_individual(self.problem, inds)
+                prob = best.production_probabilities(
+                    lambda x: self.representation.genotype_to_phenotype(
+                        self.grammar,
+                        x,
+                    ),
+                    self.grammar,
+                )
+                self.grammar = self.grammar.update_weights(
+                    self.evolve_grammar.learning_rate,
+                    prob,
+                )
+                best_overall = not best_overall
             gen += 1
         self.final_population = population
 
@@ -345,7 +375,22 @@ class GP(Heuristics):
             ),
         )
 
-    def init_population(self):
-        return [
-            self.create_individual(self.max_depth) for _ in range(self.population_size)
-        ]
+    def init_population(self, ramped_half_and_half):
+        if ramped_half_and_half:
+            n_not_ramped = int(self.population_size / 2)
+            n_ramped = self.population_size - n_not_ramped 
+            pop_ramped = [
+                self.create_individual(max((i % self.max_init_depth) + 1, self.grammar.get_min_tree_depth()))
+                for i in range(n_ramped)
+            ]
+            pop_not_ramped = [
+                self.create_individual(self.max_init_depth)
+                for _ in range(n_not_ramped)
+            ]
+                
+            return pop_ramped + pop_not_ramped
+        else:
+            return [
+                self.create_individual(self.max_init_depth)
+                for _ in range(self.population_size)
+            ]
