@@ -15,6 +15,7 @@ from typing import Set
 from typing import Tuple
 from typing import Type
 
+from geneticengine.core.decorators import get_gengy
 from geneticengine.core.utils import all_init_arguments_typed
 from geneticengine.core.utils import get_arguments
 from geneticengine.core.utils import get_generic_parameter
@@ -29,11 +30,23 @@ from geneticengine.core.utils import strip_annotations
 
 
 class InvalidGrammarException(Exception):
-    """Exception to be raised when a passed node is neither abstract nor typed."""
+    """Exception to be raised when a passed node is neither abstract nor
+    typed."""
 
     pass
 
 
+class EvolveGrammar:
+    """Class to be assigned to the evolve_grammar parameter in the GP class.
+
+    Args:
+        learning_rate (float): The learning rate with which the grammar is updated each generation (default as in https://arxiv.org/pdf/2103.08389.pdf = 0.01).
+    """
+
+    def __init__(self, learning_rate=0.01) -> None:
+        self.learning_rate = learning_rate
+
+
 class Grammar:
     starting_symbol: type
     alternatives: dict[type, list[type]]
@@ -90,10 +103,8 @@ class Grammar:
                 )
 
     def register_alternative(self, nonterminal: type, nodetype: type):
-        """
-        Register a production A->B
-        Call multiple times with same A to register many possible alternatives.
-        """
+        """Register a production A->B Call multiple times with same A to
+        register many possible alternatives."""
         if not is_abstract(nonterminal):
             raise Exception(
                 f"Trying to register an alternative on a non-abstract class: {nonterminal} -> {nodetype}\n"
@@ -162,10 +173,15 @@ class Grammar:
             return n
 
         def format(x):
+            def add_weight(prod):
+                if "weight" in get_gengy(prod):
+                    return f'<{get_gengy(prod)["weight"]}>'
+                return ""
+
             args = ", ".join(
                 [f"{a}: {wrap(at)}" for (a, at) in get_arguments(x)],
             )
-            return f"{x.__name__}({args})"
+            return f"{x.__name__}({args}){add_weight(x)}"
 
         prods = ";".join(
             [
@@ -180,13 +196,13 @@ class Grammar:
         )
 
     def get_all_symbols(self) -> tuple[set[type], set[type], set[type]]:
-        """All symbols in the current grammar, including terminals"""
+        """All symbols in the current grammar, including terminals."""
         keys = {k for k in self.alternatives.keys()}
         sequence = {v for vv in self.alternatives.values() for v in vv}
         return (keys, sequence, sequence.union(keys).union(self.all_nodes))
 
     def get_distance_to_terminal(self, ty: type) -> int:
-        """Returns the current distance to terminal of a given type"""
+        """Returns the current distance to terminal of a given type."""
         if is_annotated(ty):
             ta = get_generic_parameter(ty)
             return self.get_distance_to_terminal(ta)
@@ -201,11 +217,11 @@ class Grammar:
             return self.distanceToTerminal[ty]
 
     def get_min_tree_depth(self):
-        """Returns the minimum depth a tree must have"""
+        """Returns the minimum depth a tree must have."""
         return self.distanceToTerminal[self.starting_symbol]
 
     def get_max_node_depth(self):
-        """Returns the maximum minimum depth a node can have"""
+        """Returns the maximum minimum depth a node can have."""
 
         def dist(x):
             return self.distanceToTerminal[x]
@@ -296,24 +312,56 @@ class Grammar:
             else:
                 pass
 
+    def get_weights(self):
+        weights = {prod: get_gengy(prod).get("weight", 1.0) for prod in self.all_nodes}
+        return weights
+
+    def update_weights(self, learning_rate, extra_weights):
+        weights = self.get_weights()
+        for rule in self.alternatives:
+            prods = self.alternatives[rule]
+            total_weights = 0
+            for prod in prods:
+                weights[prod] += learning_rate * extra_weights[prod]
+                total_weights += weights[prod]
+            for prod in prods:
+                weights[prod] = weights[prod] / total_weights
+
+        for weight in weights:
+            assert weights[weight] >= 0 and weights[weight] <= 1
+
+        starting_symbol = self.starting_symbol
+        starting_symbol.__dict__["__gengy__"]["weight"] = weights[starting_symbol]
+        nodes = list()
+        for node in self.considered_subtypes:
+            node.__dict__["__gengy__"]["weight"] = weights[node]
+            nodes.append(node)
+        self.__init__(starting_symbol, nodes, self.expansion_depthing)
+        self.register_type(starting_symbol)
+        self.preprocess()
+        return self
+
 
 def extract_grammar(
     considered_subtypes: list[type],
     starting_symbol: type,
     expansion_depthing: bool = False,
 ):
-    """
-    The extract_grammar takes in all the productions of the grammar (nodes) and a starting symbol (starting_symbol). It goes through all the nodes and constructs a complete grammar that can then be used for search algorithms such as Genetic Programming and Hill Climbing.
+    """The extract_grammar takes in all the productions of the grammar (nodes)
+    and a starting symbol (starting_symbol). It goes through all the nodes and
+    constructs a complete grammar that can then be used for search algorithms
+    such as Genetic Programming and Hill Climbing.
 
-    Parameters:
-        - nodes (list): A list of objects representing tree nodes. Make sure that any node can be produced be the starting symbol.
-        - starting_symbol (object): The starting symbol of each tree. Makes sure every generated tree by the returned grammar starts with this symbol. Make sure that the starting symbol can produce any object of nodes.
+    Args:
+        nodes (list): A list of objects representing tree nodes. Make sure that any node can be produced be the starting symbol.
+        starting_symbol (object): The starting symbol of each tree. Makes sure every generated tree by the returned grammar starts with this symbol. Make sure that the starting symbol can produce any object of nodes.
 
     Returns:
-        - The grammar
-
+        The grammar
     """
     g = Grammar(starting_symbol, considered_subtypes, expansion_depthing)
     g.register_type(starting_symbol)
     g.preprocess()
+    if any(["weight" in get_gengy(p) for p in considered_subtypes]):
+        g.update_weights(1, g.get_weights())
     return g
