@@ -14,6 +14,8 @@ from sklearn.base import TransformerMixin
 from geneticengine.algorithms.gp.gp import GP
 from geneticengine.algorithms.hill_climbing import HC
 from geneticengine.core.grammar import extract_grammar
+from geneticengine.core.problems import Problem
+from geneticengine.core.problems import SingleObjectiveProblem
 from geneticengine.core.random.sources import RandomSource
 from geneticengine.core.representations.api import Representation
 from geneticengine.core.representations.tree.treebased import treebased_representation
@@ -35,23 +37,23 @@ from geneticengine.off_the_shelf.sympy_compatible import fix_all
 class GeneticProgrammingClassifier(BaseEstimator, TransformerMixin):
     """
     Genetic Programming Classifier. Main attributes: fit and predict
+    Defaults as given in A Field Guide to GP, p.17, by Poli and Mcphee:
+
+    Args:
+        nodes (List[Number]): The list of nodes to be used in the grammar. You can design your own, or use the ones in geneticengine.grammars.[sgp,literals,basic_math]. The default uses [ Plus, Mul, ExpLiteral, Var, SafeDiv, SafeLog, SafeSqrt ] + exp_literals.
+        representation (Representation): The individual representation used by the GP program. The default is treebased_representation. Currently Genetic Engine also supports Grammatical Evolution: geneticengine.core.representations.grammatical_evolution.ge_representation. You can also deisgn your own.
+        seed (int): The seed for the RandomSource (default = 123).
+        population_size (int): The population size (default = 200). Apart from the first generation, each generation the population is made up of the elites, novelties, and transformed individuals from the previous generation. Note that population_size > (n_elites + n_novelties + 1) must hold.
+        n_elites (int): Number of elites, i.e. the number of best individuals that are preserved every generation (default = 5).
+        n_novelties (int): Number of novelties, i.e. the number of newly generated individuals added to the population each generation. (default = 10).
+        number_of_generations (int): Number of generations (default = 100).
+        max_depth (int): The maximum depth a tree can have (default = 15).
+        favor_less_deep_trees (bool): If set to True, this gives a tiny penalty to deeper trees to favor simpler trees (default = False).
+        hill_climbing (bool): Allows the user to change the standard mutation operations to the hill-climbing mutation operation, in which an individual is mutated to 5 different new individuals, after which the best is chosen to survive (default = False).
+
+        probability_mutation (float): probability that an individual is mutated (default = 0.01).
+        probability_crossover (float): probability that an individual is chosen for cross-over (default = 0.9).
 
-    Parameters:
-        - nodes (List[Number]): The list of nodes to be used in the grammar. You can design your own, or use the ones in geneticengine.grammars.[sgp,literals,basic_math]. The default uses [ Plus, Mul, ExpLiteral, Var, SafeDiv, SafeLog, SafeSqrt ] + exp_literals.
-        - representation (Representation): The individual representation used by the GP program. The default is treebased_representation. Currently Genetic Engine also supports Grammatical Evolution: geneticengine.core.representations.grammatical_evolution.ge_representation. You can also deisgn your own.
-        - seed (int): The seed for the RandomSource (default = 123).
-        - population_size (int): The population size (default = 200). Apart from the first generation, each generation the population is made up of the elites, novelties, and transformed individuals from the previous generation. Note that population_size > (n_elites + n_novelties + 1) must hold.
-        - n_elites (int): Number of elites, i.e. the number of best individuals that are preserved every generation (default = 5).
-        - n_novelties (int): Number of novelties, i.e. the number of newly generated individuals added to the population each generation. (default = 10).
-        - number_of_generations (int): Number of generations (default = 100).
-        - max_depth (int): The maximum depth a tree can have (default = 15).
-        - favor_less_deep_trees (bool): If set to True, this gives a tiny penalty to deeper trees to favor simpler trees (default = False).
-        - hill_climbing (bool): Allows the user to change the standard mutation operations to the hill-climbing mutation operation, in which an individual is mutated to 5 different new individuals, after which the best is chosen to survive (default = False).
-        -----
-        Defaults as given in A Field Guide to GP, p.17, by Poli and Mcphee:
-        - probability_mutation (float): probability that an individual is mutated (default = 0.01).
-        - probability_crossover (float): probability that an individual is chosen for cross-over (default = 0.9).
-        -----
     """
 
     def __init__(
@@ -147,7 +149,11 @@ class GeneticProgrammingClassifier(BaseEstimator, TransformerMixin):
 
         model = GP(
             grammar=self.grammar,
-            evaluation_function=fitness_function,
+            problem=SingleObjectiveProblem(
+                minimize=False,
+                fitness_function=fitness_function,
+                target_fitness=None,
+            ),
             representation=self.representation,
             population_size=self.population_size,
             n_elites=self.n_elites,
@@ -164,7 +170,6 @@ class GeneticProgrammingClassifier(BaseEstimator, TransformerMixin):
         best_ind, fitness, phenotype = model.evolve(verbose=1)
         self.evolved_phenotype = phenotype
         self.sympy_compatible_phenotype = fix_all(str(phenotype))
-        
 
     def predict(self, X):
         """
@@ -192,13 +197,13 @@ class HillClimbingClassifier(BaseEstimator, TransformerMixin):
     """
     Hill Climbing Classifier. Main attributes: fit and predict
 
-    Parameters:
-        - nodes (List[Number]): The list of nodes to be used in the grammar. You can design your own, or use the ones in geneticengine.grammars.[sgp,literals,basic_math]. The default uses [ Plus, Mul, ExpLiteral, Var, SafeDiv, SafeLog, SafeSqrt ] + exp_literals.
-        - representation (Representation): The individual representation used by the GP program. The default is treebased_representation. Currently Genetic Engine also supports Grammatical Evolution: geneticengine.core.representations.grammatical_evolution.ge_representation. You can also deisgn your own.
-        - seed (int): The seed for the RandomSource (default = 123).
-        - population_size (int): The population size (default = 200). Apart from the first generation, each generation the population is made up of the elites, novelties, and transformed individuals from the previous generation. Note that population_size > (n_elites + n_novelties + 1) must hold.
-        - number_of_generations (int): Number of generations (default = 100).
-        - max_depth (int): The maximum depth a tree can have (default = 15).
+    Args:
+        nodes (List[Number]): The list of nodes to be used in the grammar. You can design your own, or use the ones in geneticengine.grammars.[sgp,literals,basic_math]. The default uses [ Plus, Mul, ExpLiteral, Var, SafeDiv, SafeLog, SafeSqrt ] + exp_literals.
+        representation (Representation): The individual representation used by the GP program. The default is treebased_representation. Currently Genetic Engine also supports Grammatical Evolution: geneticengine.core.representations.grammatical_evolution.ge_representation. You can also deisgn your own.
+        seed (int): The seed for the RandomSource (default = 123).
+        population_size (int): The population size (default = 200). Apart from the first generation, each generation the population is made up of the elites, novelties, and transformed individuals from the previous generation. Note that population_size > (n_elites + n_novelties + 1) must hold.
+        number_of_generations (int): Number of generations (default = 100).
+        max_depth (int): The maximum depth a tree can have (default = 15).
     """
 
     def __init__(
@@ -276,7 +281,11 @@ class HillClimbingClassifier(BaseEstimator, TransformerMixin):
 
         model = HC(
             g=self.grammar,
-            evaluation_function=fitness_function,
+            problem=SingleObjectiveProblem(
+                minimize=False,
+                fitness_function=fitness_function,
+                target_fitness=None,
+            ),
             representation=self.representation,
             population_size=self.population_size,
             number_of_generations=self.number_of_generations,
@@ -308,4 +317,3 @@ class HillClimbingClassifier(BaseEstimator, TransformerMixin):
         y_pred = self.evolved_phenotype.evaluate(**variables)
 
         return y_pred
-
